/*
 * (C) Copyright 2010
 * Allied Telesis <www.alliedtelesis.com>
 */

/* based (loosley) on Marvell's board/mv_feroceon/mv_kw/platform.S */

#include <config.h>
#include <asm/arch/kirkwood.h>

platform_lowlevel_init_data:
	/* AHB to Mbus Bridge
	 * Change CPU reg base to 0xF1000000
	 *  These *MUST* be first, and it hits
	 *  both potential address to be safe
	 */
	.long 0xD0020080, KW_REGS_PHY_BASE
	.long 0xF1020080, KW_REGS_PHY_BASE

	/* Configure RGMII-0 interface pad voltage to 1.8V */
	.long KW_REGS_PHY_BASE + 0x100E0, 0x1B1B1B1B

	.long KW_REGS_PHY_BASE + 0x20134, 0xffffffff
	.long KW_REGS_PHY_BASE + 0x20138, 0x009fffff
	.long KW_REGS_PHY_BASE + 0x20154, 0x00000000
	.long KW_REGS_PHY_BASE + 0x2014c, 0x00000000
	.long KW_REGS_PHY_BASE + 0x20148, 0x00000001

	/* Dram initalization for 1 x x16
	 *  DDR II Micron part number MT47H64M16HR-3
	 *  MClk 333MHz, Size 128MB, ECC disable
	 */
	.long KW_REGS_PHY_BASE + 0x01400, 0x43000618    /* DDR Configuration register */
	.long KW_REGS_PHY_BASE + 0x01404, 0x35143000	/* DDR Controller Control Low */
	.long KW_REGS_PHY_BASE + 0x01408, 0x11012227	/* DDR Timing (Low) (active cycles value +1) */
	.long KW_REGS_PHY_BASE + 0x0140C, 0x00000819	/* DDR Timing (High) */
	.long KW_REGS_PHY_BASE + 0x01410, 0x0000000d	/* DDR Address Control */
	.long KW_REGS_PHY_BASE + 0x01414, 0x00000000	/* DDR Open Pages Control */
	.long KW_REGS_PHY_BASE + 0x01418, 0x00000000	/* DDR Operation */
	.long KW_REGS_PHY_BASE + 0x0141C, 0x00000632	/* DDR Mode */
	.long KW_REGS_PHY_BASE + 0x01420, 0x00000040	/* DDR Extended Mode */
	.long KW_REGS_PHY_BASE + 0x01424, 0x0000f07f	/* DDR Controller Control High */
	.long KW_REGS_PHY_BASE + 0x01428, 0x00063300	/* DDR2 ODT Read Timing (default values) */
	.long KW_REGS_PHY_BASE + 0x0147C, 0x00006330	/* DDR2 ODT Write Timing (default values) */

	.long KW_REGS_PHY_BASE + 0x01500, 0x00000000	/* SDRAM CS[0] Base address at 0x00000000 */
	.long KW_REGS_PHY_BASE + 0x01504, 0x07FFFFF1	/* SDRAM CS[0] Size 128MiB */
	.long KW_REGS_PHY_BASE + 0x01508, 0x10000000	/* SDRAM CS[1] Base address */
	.long KW_REGS_PHY_BASE + 0x0150C, 0x00FFFFF4	/* SDRAM CS[1] Size, window disabled */
	.long KW_REGS_PHY_BASE + 0x01510, 0x20000000	/* SDRAM CS[2] Base address */
	.long KW_REGS_PHY_BASE + 0x01514, 0x00FFFFF8	/* SDRAM CS[2] Size, window disabled */
	.long KW_REGS_PHY_BASE + 0x01518, 0x30000000	/* SDRAM CS[3] Base address */
	.long KW_REGS_PHY_BASE + 0x0151C, 0x00FFFFFC	/* SDRAM CS[3] Size, window disabled */

	.long KW_REGS_PHY_BASE + 0x01494, 0x00030000	/* DDR ODT Control (Low) */
	.long KW_REGS_PHY_BASE + 0x01498, 0x00000000	/* DDR ODT Control (High) */
	.long KW_REGS_PHY_BASE + 0x0149C, 0x0000e803	/* CPU ODT Control */

	.long KW_REGS_PHY_BASE + 0x01480, 0x00000001	/* DDR Initialization Control */

	/* End of Header extension */
	.long 0x0, 0x0

.globl lowlevel_init

/************************************************/
/*              lowlevel_init                   *
/************************************************/

lowlevel_init:
        /* save Link Registers */
	mov	r2, lr

	/* change CPU registers according to platform spec */
	adr	r4, platform_lowlevel_init_data
	b	check_next

reg_write:
	/* TODO: This does not yet cater for BigEndian (BE) systems */
	str	r1, [r0]
	add	r4, #8
check_next:
	ldr	r0, [r4]
	ldr	r1, [r4, #4]
	cmp	r0, #0x0
	bne	reg_write

	/* Change L2 cache to exist */
	ldr	r1, =(KW_REGS_PHY_BASE + 0x20128)
	ldr	r4, [r1]
	orr     r4, r4, #0x18
	str	r4, [r1]

	/* Read operation to make sure the L2 bit is set */
	ldr	r4, [r1]

	/* invalidate L2 cache */
	mov	r0, #0
	mcr	p15, 1, r0, c15, c11, 0

#if defined(CONFIG_SPI_FLASH)
	/* configure the Prescale of SPI clk Tclk = 166MHz */
	ldr	r1, =(KW_REGS_PHY_BASE + 0x10604)
	ldr	r6, [r1]
	and	r6, r6, #~0x1f
	orr	r6, r6, #0x15
	str	r6, [r1]
#endif

done:
	mov	lr, r2
	mov pc, lr
