 /*
 * Copyright 2004 Freescale Semiconductor.
 * (C) Copyright 2002,2003, Motorola Inc.
 * Xianghua Xiao, (X.Xiao@motorola.com)
 *
 * (C) Copyright 2002 Scott McNutt <smcnutt@artesyncp.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <common.h>
#include <pci.h>
#include <asm/processor.h>
#include <asm/immap_85xx.h>
#include <ioports.h>
#include <spd.h>


#if defined(CONFIG_DDR_ECC)
extern void ddr_enable_ecc(unsigned int dram_size);
#endif

extern long int spd_sdram(void);

void local_bus_init(void);
void sdram_init(void);
long int fixed_sdram(void);

/*
 * I/O Port configuration table
 *
 * if conf is 1, then that port pin will be configured at boot time
 * according to the five values podr/pdir/ppar/psor/pdat for that entry
 */

const iop_conf_t iop_conf_tab[4][32] = {

    /* Port A configuration */
    {   /*            conf ppar psor pdir podr pdat */
	/* PA31 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1 TxENB */
	/* PA30 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1 TxClav   */
	/* PA29 */ {   1,   1,   1,   1,   0,   0   }, /* FCC1 TxSOC  */
	/* PA28 */ {   1,   1,   1,   1,   0,   0   }, /* FCC1 RxENB */
	/* PA27 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1 RxSOC */
	/* PA26 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 RxClav */
	/* PA25 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMTXD[0] */
	/* PA24 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMTXD[1] */
	/* PA23 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMTXD[2] */
	/* PA22 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMTXD[3] */
	/* PA21 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[4] */
	/* PA20 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[5] */
	/* PA19 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[6] */
	/* PA18 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1 ATMTXD[7] */
	/* PA17 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[7] */
	/* PA16 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[6] */
	/* PA15 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[5] */
	/* PA14 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1 ATMRXD[4] */
	/* PA13 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMRXD[3] */
	/* PA12 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMRXD[2] */
	/* PA11 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMRXD[1] */
	/* PA10 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMRXD[0] */
	/* PA9  */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 L1TXD */
	/* PA8  */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 L1RXD */
	/* PA7  */ {   1,   0,   0,   0,   0,   0   }, /* PA7 */
	/* PA6  */ {   1,   0,   0,   0,   0,   0   }, /* TDM A1 L1RSYNC */
	/* PA5  */ {   1,   0,   0,   0,   0,   0   }, /* PA5 */
	/* PA4  */ {   1,   0,   0,   0,   0,   0   }, /* PA4 */
	/* PA3  */ {   1,   0,   0,   0,   0,   0   }, /* PA3 */
	/* PA2  */ {   1,   0,   0,   0,   0,   0   }, /* PA2 */
	/* PA1  */ {   1,   0,   0,   0,   0,   0   }, /* FREERUN */
	/* PA0  */ {   1,   0,   0,   0,   0,   0   }  /* PA0 */
    },

    /* Port B configuration */
    {   /*            conf ppar psor pdir podr pdat */
	/* PB31 */ {   1,   0,   0,   1,   0,   0   }, /* FCC2 MII TX_ER */
	/* PB30 */ {   1,   0,   0,   1,   0,   0   }, /* FCC2 MII RX_DV */
	/* PB29 */ {   1,   0,   0,   0,   0,   0   }, /* FCC2 MII TX_EN */
	/* PB28 */ {   1,   1,   1,   1,   0,   0   }, /* FCC2 MII RX_ER */
	/* PB27 */ {   1,   1,   1,   0,   0,   0   }, /* FCC2 MII COL */
	/* PB26 */ {   1,   1,   1,   0,   0,   0   }, /* FCC2 MII CRS */
	/* PB25 */ {   1,   0,   0,   0,   0,   0   }, /* FCC2 MII TxD[3] */
	/* PB24 */ {   1,   1,   1,   0,   0,   0   }, /* FCC2 MII TxD[2] */
	/* PB23 */ {   1,   0,   0,   0,   0,   0   }, /* FCC2 MII TxD[1] */
	/* PB22 */ {   1,   0,   0,   0,   0,   0   }, /* FCC2 MII TxD[0] */
	/* PB21 */ {   1,   0,   0,   1,   0,   0   }, /* FCC2 MII RxD[0] */
	/* PB20 */ {   1,   0,   0,   0,   0,   0   }, /* FCC2 MII RxD[1] */
	/* PB19 */ {   1,   0,   0,   0,   0,   0   }, /* FCC2 MII RxD[2] */
	/* PB18 */ {   1,   0,   0,   1,   0,   0   }, /* FCC2 MII RxD[3] */
	/* PB17 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:RX_DIV */
	/* PB16 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:RX_ERR */
	/* PB15 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:TX_ERR */
	/* PB14 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:TX_EN */
	/* PB13 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:COL */
	/* PB12 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:CRS */
	/* PB11 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB10 */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB9  */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB8  */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:RXD */
	/* PB7  */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:TXD */
	/* PB6  */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:TXD */
	/* PB5  */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:TXD */
	/* PB4  */ {   1,   0,   0,   0,   0,   0   }, /* FCC3:TXD */
	/* PB3  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PB2  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PB1  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PB0  */ {   0,   0,   0,   0,   0,   0   }  /* pin doesn't exist */
    },

    /* Port C */
    {   /*            conf ppar psor pdir podr pdat */
	/* PC31 */ {   1,   0,   0,   0,   0,   0   }, /* PC31 */
	/* PC30 */ {   1,   0,   0,   0,   0,   0   }, /* PC30 */
	/* PC29 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1 EN *CLSN */
	/* PC28 */ {   1,   1,   0,   0,   0,   0   }, /* PC28 */
	/* PC27 */ {   1,   1,   0,   0,   0,   0   }, /* UART Clock in */
	/* PC26 */ {   1,   1,   0,   0,   0,   0   }, /* PC26 */
	/* PC25 */ {   1,   0,   0,   1,   0,   0   }, /* PC25 */
	/* PC24 */ {   1,   0,   0,   0,   0,   0   }, /* PC24 */
	/* PC23 */ {   1,   1,   0,   0,   0,   0   }, /* ATMTFCLK */
	/* PC22 */ {   1,   0,   0,   0,   0,   0   }, /* ATMRFCLK */
	/* PC21 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1 EN RXCLK */
	/* PC20 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1 EN TXCLK */
	/* PC19 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2 MII RX_CLK CLK13 */
	/* PC18 */ {   1,   0,   0,   1,   0,   0   }, /* FCC Tx Clock (CLK14) */
	/* PC17 */ {   1,   0,   0,   1,   0,   0   }, /* PC17 */
	/* PC16 */ {   1,   0,   0,   1,   0,   0   }, /* FCC Tx Clock (CLK16) */
	/* PC15 */ {   1,   1,   0,   0,   0,   1   }, /* PC15 */
	/* PC14 */ {   1,   1,   0,   0,   0,   1   }, /* SCC1 EN *CD */
	/* PC13 */ {   1,   0,   0,   0,   0,   1   }, /* PC13 */
	/* PC12 */ {   1,   0,   0,   1,   0,   1   }, /* PC12 */
	/* PC11 */ {   1,   1,   0,   0,   0,   1   }, /* LXT971 transmit control */
	/* PC10 */ {   1,   1,   0,   0,   0,   1   }, /* FETHMDC */
	/* PC9  */ {   1,   0,   0,   1,   0,   0   }, /* FETHMDIO */
	/* PC8  */ {   1,   0,   0,   1,   0,   0   }, /* PC8 */
	/* PC7  */ {   1,   0,   0,   1,   0,   0   }, /* PC7 */
	/* PC6  */ {   1,   0,   0,   1,   0,   0   }, /* PC6 */
	/* PC5  */ {   1,   0,   0,   1,   0,   0   }, /* PC5 */
	/* PC4  */ {   1,   0,   0,   1,   0,   0   }, /* PC4 */
	/* PC3  */ {   1,   0,   0,   0,   0,   0   }, /* PC3 */
	/* PC2  */ {   1,   0,   0,   0,   0,   0   }, /* ENET FDE */
	/* PC1  */ {   1,   0,   0,   1,   0,   0   }, /* ENET DSQE */
	/* PC0  */ {   1,   0,   0,   1,   0,   0   }, /* ENET LBK */
    },

    /* Port D */
    {   /*            conf ppar psor pdir podr pdat */
	/* PD31 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1 EN RxD */
	/* PD30 */ {   1,   0,   0,   0,   0,   0   }, /* SCC1 EN TxD */
	/* PD29 */ {   1,   1,   0,   1,   0,   1   }, /* SCC1 EN TENA */
	/* PD28 */ {   1,   0,   0,   0,   0,   0   }, /* PD28 */
	/* PD27 */ {   1,   0,   0,   0,   0,   0   }, /* PD27 */
	/* PD26 */ {   1,   0,   0,   0,   0,   0   }, /* PD26 */
	/* PD25 */ {   1,   1,   0,   0,   0,   0   }, /* PD25 */
	/* PD24 */ {   1,   1,   0,   1,   0,   0   }, /* PD24 */
	/* PD23 */ {   1,   1,   0,   1,   0,   1   }, /* PD23 */
	/* PD22 */ {   1,   1,   1,   0,   0,   0   }, /* PD22 */
	/* PD21 */ {   1,   1,   1,   0,   0,   0   }, /* PD21 */
	/* PD20 */ {   1,   1,   1,   0,   0,   0   }, /* PD20 */
	/* PD19 */ {   1,   0,   0,   0,   0,   0   }, /* PD19 */
	/* PD18 */ {   1,   0,   0,   1,   0,   0   }, /* PD18 */
	/* PD17 */ {   1,   0,   0,   1,   0,   0   }, /* FCC1 ATMRXPRTY */
	/* PD16 */ {   1,   0,   0,   0,   0,   0   }, /* FCC1 ATMTXPRTY */
	/* PD15 */ {   1,   0,   0,   0,   1,   0   }, /* I2C SDA */
	/* PD14 */ {   1,   0,   0,   1,   0,   0   }, /* LED */
	/* PD13 */ {   1,   0,   0,   1,   0,   0   }, /* PD13 */
	/* PD12 */ {   1,   0,   0,   0,   0,   0   }, /* PD12 */
	/* PD11 */ {   1,   0,   0,   0,   0,   0   }, /* PD11 */
	/* PD10 */ {   1,   0,   0,   0,   0,   0   }, /* PD10 */
	/* PD9  */ {   1,   0,   0,   0,   0,   0   }, /* SMC1 TXD */
	/* PD8  */ {   1,   0,   0,   0,   0,   0   }, /* SMC1 RXD */
	/* PD7  */ {   1,   0,   0,   0,   0,   0   }, /* PD7 */
	/* PD6  */ {   1,   0,   0,   0,   0,   0   }, /* PD6 */
	/* PD5  */ {   1,   0,   0,   0,   0,   0   }, /* PD5 */
	/* PD4  */ {   1,   0,   0,   0,   0,   0   }, /* PD4 */
	/* PD3  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PD2  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PD1  */ {   0,   0,   0,   0,   0,   0   }, /* pin doesn't exist */
	/* PD0  */ {   0,   0,   0,   0,   0,   0   }  /* pin doesn't exist */
    }
};


int board_early_init_f (void)
{
#if defined(CONFIG_PCI)
    volatile immap_t *immr = (immap_t *)CFG_IMMR;
    volatile ccsr_pcix_t *pci = &immr->im_pcix;

    pci->peer &= 0xffffffdf; /* disable master abort */
#endif

    return 0;
}

#ifdef CONFIG_MISC_INIT_R
int misc_init_r (void)
{
    volatile immap_t *immap = (immap_t *)CFG_IMMR;
    volatile ccsr_cpm_t *cpm2= &immap->im_cpm;

    /* disable the phy */
    cpm2->im_cpm_iop.pdata &= ~0x00200000;
    udelay(1000); 
    /* enable the phy */
    cpm2->im_cpm_iop.pdata |= 0x00200000;
    udelay(10);


  /* turn off the fault LED */

//   *((char*)(CFG_EPLD_BASE+0x42)) = 0x01;
// 
// 	/* Enable the PHY LED buffer */
// 	*((char*)(CFG_EPLD_BASE+0xaf)) = 0x01;
// 
// 	/* Bring the management PHY out of reset */
// 	*((char*)(CFG_EPLD_BASE+0xae)) = 0x01;

	/* Sleep for 0.5sec to allow PHY to come out of reset */
	//udelay(500000);

	return 0;
}
#endif

int checkboard (void)
{
#ifdef CONFIG_ALLIEDTELESIS_OUTPUT
	if (!console_full_output())
        {
          local_bus_init();
          return 0;
        }
#endif /* CONFIG_ALLIEDTELESIS_OUTPUT */
#ifdef CONFIG_PCI
	printf ("Board: ATR FreeScale MPC85xx\n");
 	printf("    PCI1: 32 bit, %d MHz (compiled)\n",
 	       			CONFIG_SYS_CLK_FREQ / 1000000);
 	printf("    PCI2: 32 bit, %d MHz (compiled)\n",
 	       			CONFIG_SYS_CLK_FREQ / 1000000);

#else
	puts("Board: ATL FreeScale MPC85xx\n");
	printf("    PCI1: disabled\n");
	printf("    PCI2: disabled\n");
#endif
	/*
	 * Initialize local bus.
	 */
	local_bus_init();

	return 0;
}


long int
initdram(int board_type)
{
	long dram_size = 0;
	extern long spd_sdram (void);
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
#ifdef CONFIG_ALLIEDTELESIS_OUTPUT
	if (console_full_output())
#endif /* CONFIG_ALLIEDTELESIS_OUTPUT */
	puts("Initializing\n");

#if defined(CONFIG_DDR_DLL)
	{
		/*
		 * Work around to stabilize DDR DLL MSYNC_IN.
		 * Errata DDR9 seems to have been fixed.
		 * This is now the workaround for Errata DDR11:
		 *    Override DLL = 1, Course Adj = 1, Tap Select = 0
		 */

		volatile ccsr_gur_t *gur= &immap->im_gur;

		gur->ddrdllcr = 0x81000000;
		asm("sync;isync;msync");
		udelay(200);
	}
#endif

#if defined(CONFIG_SPD_EEPROM)
	dram_size = spd_sdram ();
#else
	dram_size = fixed_sdram ();
#endif

#if defined(CONFIG_DDR_ECC)
	/*
	 * Initialize and enable DDR ECC.
	 */
	ddr_enable_ecc(dram_size);
#endif

	/*
	 * Initialize SDRAM.
	 */
	sdram_init();
#ifdef CONFIG_ALLIEDTELESIS_OUTPUT
	if (!console_full_output())
          puts("DDR: ");
        else
#endif /* CONFIG_ALLIEDTELESIS_OUTPUT */
	puts("    DDR: ");
	return dram_size;
}


/*
 * Initialize Local Bus
 */

void
local_bus_init(void)
{
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	volatile ccsr_gur_t *gur = &immap->im_gur;
	volatile ccsr_lbc_t *lbc = &immap->im_lbc;

	uint clkdiv;
	uint lbc_hz;
	sys_info_t sysinfo;

	/*
	 * Errata LBC11.
	 * Fix Local Bus clock glitch when DLL is enabled.
	 *
	 * If localbus freq is < 66Mhz, DLL bypass mode must be used.
	 * If localbus freq is > 133Mhz, DLL can be safely enabled.
	 * Between 66 and 133, the DLL is enabled with an override workaround.
	 */

	get_sys_info(&sysinfo);
	clkdiv = lbc->lcrr & 0x0f;
	lbc_hz = sysinfo.freqSystemBus / 1000000 / clkdiv;

	if (lbc_hz < 66) {
		lbc->lcrr = CFG_LBC_LCRR | 0x80000000;	/* DLL Bypass */

	} else if (lbc_hz >= 133) {
		lbc->lcrr = CFG_LBC_LCRR & (~0x80000000); /* DLL Enabled */

	} else {
		/*
		 * On REV1 boards, need to change CLKDIV before enable DLL.
		 * Default CLKDIV is 8, change it to 4 temporarily.
		 */
		uint pvr = get_pvr();
		uint temp_lbcdll = 0;

		if (pvr == PVR_85xx_REV1) {
			/* FIXME: Justify the high bit here. */
			lbc->lcrr = 0x10000004;
		}

		lbc->lcrr = CFG_LBC_LCRR & (~0x80000000); /* DLL Enabled */
		udelay(200);

		/*
		 * Sample LBC DLL ctrl reg, upshift it to set the
		 * override bits.
		 */
		temp_lbcdll = gur->lbcdllcr;
		gur->lbcdllcr = (((temp_lbcdll & 0xff) << 16) | 0x80000000);
		asm("sync;isync;msync");
	}
}


/*
 * Initialize SDRAM memory on the Local Bus.
 */

void
sdram_init(void)
{
#if CFG_LBC_SDRAM_SIZE == 0
#ifdef CONFIG_ALLIEDTELESIS_OUTPUT
	if (console_full_output())
#endif /* CONFIG_ALLIEDTELESIS_OUTPUT */
	puts("    SDRAM: N/A");
#else
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	volatile ccsr_lbc_t *lbc= &immap->im_lbc;
	uint *sdram_addr = (uint *)CFG_LBC_SDRAM_BASE;

	puts("    SDRAM: ");
	print_size (CFG_LBC_SDRAM_SIZE * 1024 * 1024, "\n");

	/*
	 * Setup SDRAM Base and Option Registers
	 */
	lbc->or2 = CFG_OR2_PRELIM;
	lbc->br2 = CFG_BR2_PRELIM;
	lbc->lbcr = CFG_LBC_LBCR;
	asm("msync");

	lbc->lsrt = CFG_LBC_LSRT;
	lbc->mrtpr = CFG_LBC_MRTPR;
	asm("sync");

	/*
	 * Configure the SDRAM controller.
	 */
	lbc->lsdmr = CFG_LBC_LSDMR_1;
	asm("sync");
	*sdram_addr = 0xff;
	ppcDcbf((unsigned long) sdram_addr);
	udelay(100);

	lbc->lsdmr = CFG_LBC_LSDMR_2;
	asm("sync");
	*sdram_addr = 0xff;
	ppcDcbf((unsigned long) sdram_addr);
	udelay(100);

	lbc->lsdmr = CFG_LBC_LSDMR_3;
	asm("sync");
	*sdram_addr = 0xff;
	ppcDcbf((unsigned long) sdram_addr);
	udelay(100);

	lbc->lsdmr = CFG_LBC_LSDMR_4;
	asm("sync");
	*sdram_addr = 0xff;
	ppcDcbf((unsigned long) sdram_addr);
	udelay(100);

	lbc->lsdmr = CFG_LBC_LSDMR_5;
	asm("sync");
	*sdram_addr = 0xff;
	ppcDcbf((unsigned long) sdram_addr);
	udelay(100);
#endif
}


#if defined(CFG_DRAM_TEST)
int testdram (void)
{
	uint *pstart = (uint *) CFG_MEMTEST_START;
	uint *pend = (uint *) CFG_MEMTEST_END;
	uint *p;

	printf("SDRAM test phase 1:\n");
	for (p = pstart; p < pend; p++)
		*p = 0xaaaaaaaa;

	for (p = pstart; p < pend; p++) {
		if (*p != 0xaaaaaaaa) {
			printf ("SDRAM test fails at: %08x\n", (uint) p);
			return 1;
		}
	}

	printf("SDRAM test phase 2:\n");
	for (p = pstart; p < pend; p++)
		*p = 0x55555555;

	for (p = pstart; p < pend; p++) {
		if (*p != 0x55555555) {
			printf ("SDRAM test fails at: %08x\n", (uint) p);
			return 1;
		}
	}

	printf("SDRAM test passed.\n");
	return 0;
}
#endif


#if !defined(CONFIG_SPD_EEPROM)
/*************************************************************************
 *  fixed sdram init -- doesn't use serial presence detect.
 ************************************************************************/
long int fixed_sdram (void)
{

  #ifndef CFG_RAMBOOT
	volatile immap_t *immap = (immap_t *)CFG_IMMR;
	volatile ccsr_ddr_t *ddr= &immap->im_ddr;

	ddr->sdram_clk_cntl = 0x82000000;
	asm("sync;isync;msync");
	udelay(500);

	ddr->cs0_bnds = CFG_DDR_CS0_BNDS;
	ddr->cs0_config = CFG_DDR_CS0_CONFIG;
	ddr->cs1_bnds = CFG_DDR_CS1_BNDS;
	ddr->cs1_config = CFG_DDR_CS1_CONFIG;
	ddr->timing_cfg_1 = CFG_DDR_TIMING_1;
	ddr->timing_cfg_2 = CFG_DDR_TIMING_2;
	ddr->sdram_mode = CFG_DDR_MODE;
	ddr->sdram_interval = CFG_DDR_INTERVAL;

    #if defined (CONFIG_DDR_ECC)
	ddr->err_disable = 0x0000000D;
	ddr->err_sbe = 0x00ff0000;
    #endif
	asm("sync;isync;msync");
	udelay(500);

    #if defined (CONFIG_DDR_ECC)
	/* Enable ECC checking */
	ddr->sdram_cfg = (CFG_DDR_CONTROL | 0x20000000);
    #else
	ddr->sdram_cfg = CFG_DDR_CONTROL;
    #endif
	asm("sync; isync; msync");
	udelay(500);
  #endif
	return CFG_SDRAM_SIZE * 1024 * 1024;
}
#endif	/* !defined(CONFIG_SPD_EEPROM) */


#if defined(CONFIG_PCI)
/*
 * Initialize PCI Devices, report devices found.
 */

#ifndef CONFIG_PCI_PNP
static struct pci_config_table pci_atr85xx_config_table[] = {
    { PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID, PCI_ANY_ID,
      PCI_IDSEL_NUMBER, PCI_ANY_ID,
      pci_cfgfunc_config_device, { PCI_ENET0_IOADDR,
				   PCI_ENET0_MEMADDR,
				   PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER
      } },
    { }
};
#endif


static struct pci_controller hose = {
#ifndef CONFIG_PCI_PNP
	config_table: pci_atr85xx_config_table,
#endif
};

#endif	/* CONFIG_PCI */


void
pci_init_board(void)
{
#ifdef CONFIG_PCI
	extern void pci_mpc85xx_init(struct pci_controller *hose);

	pci_mpc85xx_init(&hose);
#endif /* CONFIG_PCI */
}
