#ifndef _PASSWORD_H
#define _PASSWORD_H
#include <linux/types.h>
#include <bootmenu.h>

#define SEC_LEVEL1 "1"
#define SEC_LEVEL2 "2"
#define SEC_LEVEL3 "3"

#define SECURITY_LEVEL_ENV_VAR	"securityLevel"

#ifdef CONFIG_USE_PASSWORD
#ifndef CONFIG_SHA256
#error "Password needs SHA256 lib"
#endif /* CONFIG_SHA256C */
#define HASH_SIZE	64
#define CONFIG_DIAGNOSTICS /* password feature need diag code */

#define PASSWORD_MAX_LEN	23
#define PASSWORD_MIN_LEN	6
#define PASSWORD_ENV_VAR	"password"
#define	PASSWORD_HSH_LEN	32
#define BUFFER_MAX_LEN	PASSWORD_MAX_LEN+2
#define TIMEOUT_PASSWORD_PROMPT	60 /* seconds */

/* following macros keeps things simple in the rest of thecode */

#define IS_SECURITY_LEVEL(level) \
	(security_level_check(level))

//typedef uint8_t digest_t[32];

#define SEC_LEVEL1_DESCRIPTION "1 (None)"
#define SEC_LEVEL2_DESCRIPTION "2 (Password Protected)"
#define SEC_LEVEL3_DESCRIPTION "3 (Locked Down)"

#define WARNING_LEVEL1_TO_LEVEL2   \
				 "\n  WARNING: This option will set the security level to two(2)."                      \
			     "\n  Once the password has been set, some bootloader menu items"                       \
			     "\n  will require authentication."                                                     \
			     "\n  Press Y to proceed, or any other key to return to the previous menu ==> "

#define WARNING_LEVEL1_TO_LEVEL3   \
				 "\n  WARNING: This option will set the security level to three(3)."                      \
			     "\n  Once the password has been set, some bootloader menu items"                       \
			     "\n  will require authentication."                                                     \
			     "\n  Press Y to proceed, or any other key to return to the previous menu ==> "

#define WARNING_LEVEL2_TO_LEVEL3   \
				 "\n  WARNING: This option will set the security level to three(3)."                      \
				 "\n  \"Skip startup script\" menu item will no longer be available"                      \
			     "\n  Press Y to proceed, or any other key to return to the previous menu ==> "

#define WARNING_LEVEL23_TO_LEVEL1   \
				 "\n  WARNING: This option will reset the security level to One."                      \
				 "\n  which will DESTROY the entire contents of the FLASH file system"                 \
			     "\n  Press Y to proceed, or any other key to return to the previous menu ==> "

#define SECURITY_LEVEL_CHANGE(level)		\

#define PASSWORD_STATE_LOCKED ((gd->flags & GD_FLG_LOCKED_STATE) ? LOCKED : UNLOCKED)

#define SET_STATE_LOCKED  \
			(gd->flags |= GD_FLG_LOCKED_STATE)
#define SET_STATE_UNLOCKED \
			(gd->flags &= ~GD_FLG_LOCKED_STATE)

password_state_t trap_loop(void);

int protection_set(char *password, size_t password_length);

int lock_status_init(void);
password_state_t unlock_secure(char *password, size_t length);

int security_level_check(char *sec_level);
int security_level_set(char *sec_level);
char * security_level_get(void);
int password_protected_section(int *quit);

#else  /* CONFIG_USE_PASSWORD */


#define IS_SECURITY_LEVEL(level) 1

#define security_level_check(sec_level) 1

#define password_protected_section(quit) UNLOCKED


#endif



#endif				/* _PASSWORD_H */
